@@ -22,6 +22,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 #include "gnats.h"
 #include "gnatsd.h"
 #include "query.h"
+#include "db_conf.h"
 
 char *current_pr = (char *)NULL;
 
@@ -34,7 +35,9 @@ char *current_pr = (char *)NULL;
 #endif
 
 extern char myname[MAXHOSTNAMELEN];
+extern int require_db;
 extern int searching;
+extern int doret;
 extern Index *search;
 
 #ifdef HAVE_KERBEROS
@@ -91,49 +94,11 @@ lookup_pr_path (p)
   return path;
 }
 
-int
-qualify (ac, av)
-     int ac;
-     char **av;
-{
-  char pat[40];
-  int i;
-  Index *j;
-  int found = 0;
-
-  for (i = 0; i < ac; i++)
-    {
-      strcpy (pat, av[i]);
-      strcat (pat, "\\'");
-      for (j = index_chain ; j ; j = j->next)
-	if (regcmp (pat, j->number) == 0)
-	  {
-	    if (searching && !pr_matches (search, j))
-	      {
-		av[i] = (char *)NULL;
-		break;
-	      }
-	    else
-	      {
-		char *path = (char *) xmalloc (PATH_MAX);
-		sprintf (path, "%s/%s/%s", gnats_root, j->category, j->number);
-		av[i] = path;
-		found = 1;
-		/* Only go thru for a particular pattern until we make a
-		   match; once we do, move on to the next argument.  */
-		break;
-	      }
-	  }
-    }
-
-  return found;
-}
-
 int
 addr_cmp (a, b)
-     char **a, **b;
+     const void *a, *b;
 {
-  return strcmp (*a, *b);
+     return strcmp (*(char **)a, *(char **)b);
 }
 
 /* Given an array of possible addresses, make them into a real list with
@@ -149,6 +114,7 @@ unify_addrs (addrs, naddrs)
   int i, j = 0;
 
   qsort ((char *)addrs, naddrs, sizeof (char *), addr_cmp);
+    
   if (naddrs == 1)
     return 0;
 
@@ -271,7 +237,7 @@ daemon_pr (p, print)
       if (get_pr (path, p, quiet))
 	{
 	  if (print)
-	    print_pr (path, p, 1);
+	    print_pr (path, p, 1, (Index *) NULL);
 	  else
 	    {
 	      /* Even though we shouldn't do multiple loads of different
@@ -297,7 +263,8 @@ do_query (ac, av)
      int ac;
      char **av;
 {
-  int i;
+  char pat[40], *p, *n;
+  int i, opened;
   Index *j;
   int hit = 0;
 
@@ -308,36 +275,39 @@ do_query (ac, av)
       /* We weren't given a list of PRs to check, so we do the
 	 whole shooting match.  */
       for (j = index_chain; j ; j = j->next)
-	if (pr_matches (search, j))
+	if (pr_matches (search, j, &opened) && do_pr (j, opened))
 	  {
-	    if (do_pr (j))
-	      {
-		hit = 1;
-		printf ("\r\n");
-	      }
+	    hit = 1;
+	    printf ("\r\n");
 	  }
-      if (! hit)
-	printf ("%d No PRs match.\r\n", CODE_NO_PRS);
-    }
-  else if (qualify (ac, av))
-    {
-      for (i = 0; i < ac; i++)
-	{
-	  /* Note we may be clearing it out inside qualify(), if they
-	     gave a search qualifier that cancels this particular PR out.  */
-	  if (av[i])
-	    {
-	      daemon_pr (av[i], 1);
-	      hit = 1;
-	      if (! (query_format & FORMAT_FULL))
-		printf ("\r\n");
-	    }
-	}
     }
   else
+    {
+      for (i = 0; i < ac; i++)
+        {
+          p = av[i];
+          if ((n = (char *) strchr (p, '/')) != NULL) /* Remove the category */
+               p = ++n;
+          strcpy (pat, p);
+          strcat (pat, "\\'");
+          for (j = index_chain ; j ; j = j->next)
+       	    if (regcmp (pat, j->number) == 0)
+              {
+		if (pr_matches (search, j, &opened) && do_pr (j, opened))
+                  {
+                    hit = 1;
+                    printf ("\r\n");
+                  }
+                /* Only go thru for a particular pattern until we make a
+                   match; once we do, move on to the next argument.  */
+                break;
+              }
+        }
+    }
+  
+  if (! hit)
     printf ("%d No PRs match.\r\n", CODE_NO_PRS);
-
-  if (hit)
+  else
     printf (".\r\n");
 }
 
@@ -403,6 +373,15 @@ GNATS_sqlf (ac, av)
   do_query (ac, av);
 }
 
+void
+GNATS_sql2 (ac, av)
+     int ac;
+     char **av;
+{
+  query_format = FORMAT_SQL2;
+  do_query (ac, av);
+}
+
 /* Used in edit.c to force creation of a PR's text.  We'll never do that
    from here.  */
 int force = 0;
@@ -538,6 +517,202 @@ GNATS_undb (ac, av)
   printf ("%d GNATS database is now unlocked.\r\n", CODE_OK);
 }
 
+void
+GNATS_user (ac, av)
+     int ac;
+     char **av;
+{
+  char *path;
+  int   access;
+
+  if (ac == 0)
+    {
+      printf ("%d User access level set to %s\r\n", CODE_OK, access_level_str (user_access));
+      return;
+    }
+  
+  if (ac != 2)
+    {
+      printf ("%d USER requires two arguments, username and password.\r\n",
+	      CODE_ERROR);
+      return;
+    }
+  
+  path = (char *) xmalloc (PATH_MAX);
+  sprintf (path, "%s/gnats-adm/%s", gnats_root, DB_ACCESS_FILE);
+  access = get_user_access (gnats_root, path, av[0], av[1]);
+  xfree (path);
+
+  if (access < 0)
+    access = get_user_access (gnats_root, GLOBAL_ACCESS_FILE, av[0], av[1]);
+
+  if (access < 0)
+    access = ACCESS_NONE;
+  
+  if (access > host_access)
+    user_access = access;
+
+  if (user_access <= ACCESS_NONE)
+    {
+      printf ("%d You are not on the user access list: %s/%s.\r\n",
+              CODE_NO_ACCESS, av[0], av[1]);
+      exit (1);
+    }
+  
+  set_confidential_access ();
+
+  printf ("%d User access level set to %s\r\n", CODE_OK, access_level_str (user_access));
+}
+
+/* change database command */
+void
+GNATS_chdb (ac, av)
+     int ac;
+     char **av;
+{
+  static char *new_root = NULL;
+
+  char  *new_db = NULL;
+  char  *alias_db = NULL;
+  int    access;
+
+  struct stat buf;
+
+  if (! chk_nargs(ac, "Database name"))
+    return;
+
+  /* verify new root exists and is a directory */
+  new_db = av[0];
+
+  /* We only allow database aliases in gnatsd, not full pathnames */
+  alias_db = new_db;
+  new_db = lookup_conf_alias(alias_db);
+  if (!new_db)
+    {
+      printf ("%d Could not resolve database alias (%s).\r\n",
+	      CODE_FILE_ERROR, alias_db);
+      return;
+    }
+
+  if (stat(new_db, &buf) != 0)
+    {
+      if (errno == ENOENT)
+	printf ("%d Could not change to database (%s): does not exist.\r\n",
+		CODE_FILE_ERROR, new_db);
+      else
+	printf ("%d Error while getting status of (%s): errno = %d.\r\n",
+		CODE_FILE_ERROR, new_db, errno);
+      return;
+    }
+  else if ((buf.st_mode & S_IFMT) != S_IFDIR)
+    {
+      printf ("%d Specified database (%s) is not a directory.\r\n",
+	      CODE_FILE_ERROR, new_db);
+      return;
+    }
+
+  /* free existing copy then make new copy */
+  if (new_root)
+    xfree(new_root);
+  new_root = (char *) xstrdup(new_db);
+
+  /* re-initialize using new gnats_root */
+  gnats_root = new_root;
+
+  /* This doret stuff is wierd, but the following is necessary. <rickm@vsl.com> */
+  doret = 0;
+  configure  ();
+  init_gnats ();
+  doret = 1;
+
+  /* make sure that a host that is allowed to access one database does
+     not switch to a database where permission is not granted */
+  if (!allowed (current_host, current_addr, gnats_root, &access))
+    {
+      syslog (LOG_NOTICE, "%s (%s) not allowed access for %s",
+              current_host, current_addr, gnats_root);
+      printf ("%d You are not on the host access list: %s (%s).\r\n",
+              CODE_NO_ACCESS, current_host, current_addr);
+      exit (1);
+    }
+
+  /* Clear the gnatsd --require-db flag and set the user access back to the default host access */
+  require_db  = 0;
+  user_access = host_access;
+  
+  set_confidential_access ();
+  
+  free_index(index_chain);
+  index_chain = get_index ();
+/*   if (index_chain == NULL) */
+/*     { */
+/*       printf ("%d GNATS server cannot read the index.\r\n", CODE_FILE_ERROR); */
+/*       exit (1); */
+/*     } */
+
+  printf ("%d Now accessing GNATS database '%s'\r\n", CODE_OK, gnats_root);
+}
+
+/* list databases command */
+void
+GNATS_dbls (ac, av)
+     int ac;
+     char **av;
+{
+  char **databases = NULL;
+  int  count = 0;
+  int  access;
+  int i;
+
+  /* get complete list */
+  get_conf_databases(&databases, &count);
+  
+  printf ("%d List follows.\r\n", CODE_PR_READY);
+  for(i = 0; i < count; i++)
+    {
+      /* only list hosts with permitted access */
+      if (allowed (current_host, current_addr, databases[i], &access))
+	{
+	  printf ("%s\r\n", databases[i]);
+	}
+      xfree(databases[i]);
+    }
+  printf (".\r\n");
+  xfree((char*)databases);
+}
+
+/* list database alaises command */
+void
+GNATS_dbla (ac, av)
+     int ac;
+     char **av;
+{
+  char **aliases = NULL;
+  int  count = 0;
+  int  access;
+  int  i;
+
+  get_conf_aliases(&aliases, &count);
+
+  printf ("%d List follows.\r\n", CODE_PR_READY);
+  
+  for(i = 0; i < count; i++)
+    {
+      /* only list hosts with permitted access */
+      if (allowed (current_host, current_addr, lookup_conf_alias(aliases[i]), &access))
+	{
+	  /* if alias is a dot send a double dot to avoid end of data
+             confusion */
+	  if (strcmp(aliases[i], ".") == 0)
+	    printf(".");
+	  printf ("%s\r\n", aliases[i]);
+	  xfree(aliases[i]);
+	}
+    }
+  printf (".\r\n");
+  xfree((char*) aliases);
+}
+
 void
 GNATS_edit (ac, av)
      int ac;
@@ -622,6 +797,7 @@ GNATS_mlpr (ac, av)
   if (! path)
     return;
 
+  /* GNATS_mlpr must call get_pr() because of the need to get the X_GNATS_NOTIFY header */
   if (get_pr (path, av[0], 1))
     {
       Category category;
@@ -650,20 +826,20 @@ GNATS_mlpr (ac, av)
 	printf ("%d- No responsible entry for %s.\r\n",
 		CODE_INVALID_RESPONSIBLE, category.person);
 
-      addrs[naddrs++] = category.person;
+      addrs[naddrs++] = strdup (category.person);
       if (category.notify[0])
-	addrs[naddrs++] = category.notify;
+        addrs[naddrs++] = strdup (category.notify);
       if (submitter.contact[0])
-	addrs[naddrs++] = submitter.contact;
+        addrs[naddrs++] = strdup (submitter.contact);
       if (submitter.notify[0])
-	addrs[naddrs++] = submitter.notify;
+        addrs[naddrs++] = strdup (submitter.notify);
       if (responsible)
-	addrs[naddrs++] = responsible->alias;
+        addrs[naddrs++] = strdup (responsible->alias);
       h = raw_header_value (X_GNATS_NOTIFY);
       if (h)
 	{
 	  char *h2 = strchr (h, '\n'); h2[0] = '\0';
-	  addrs[naddrs++] = h;
+	  addrs[naddrs++] = strdup (h);
 	}
       addrs[naddrs] = NULL;
 
@@ -672,6 +848,8 @@ GNATS_mlpr (ac, av)
 	  free_responsible (responsible);
 	  xfree ((char *) responsible);
 	}
+      free_category  (&category);
+      free_submitter (&submitter);
 
       i = unify_addrs (addrs, naddrs);
       if (i == naddrs)
@@ -716,7 +894,8 @@ GNATS_mlct (ac, av)
   printf ("%d-Addresses to notify for category %s:\r\n",
 	  CODE_INFORMATION, av[0]);
   printf ("%d-%s\r\n", CODE_INFORMATION, category.person);
-  printf ("%d-%s\r\n", CODE_INFORMATION, category.notify);
+  if (category.notify[0])
+    printf ("%d-%s\r\n", CODE_INFORMATION, category.notify);
   printf ("%d End of list.\r\n", CODE_INFORMATION);
 }
 
@@ -825,14 +1004,11 @@ GNATS_synp (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs (ac, "synopsis"))
     return;
 
-  if (buf)
-    xfree (buf);
-  synopsis = buf = strdup(*av);
+  searching = 1;
+  search->synopsis = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -841,14 +1017,11 @@ GNATS_rlse (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs (ac, "release"))
     return;
 
-  if (buf)
-    xfree (buf);
-  release = buf = strdup(*av);
+  searching = 1;
+  search->release = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -860,8 +1033,12 @@ GNATS_conf (ac, av)
   if (! chk_nargs (ac, "yes or no"))
     return;
 
-  searching = 1;
-  search->confidential = (char *) strdup (av[0]);
+  set_confidential_access ();
+  if (!search->confidential)
+    {
+      searching = 1;
+      search->confidential = (char *) strdup (av[0]);
+    }
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -896,15 +1073,11 @@ GNATS_orig (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs (ac, "name"))
     return;
 
-  if (buf)
-    xfree (buf);
-  originator = buf = strdup(*av);
-
+  searching = 1;
+  search->originator = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -913,15 +1086,11 @@ GNATS_clss (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs(ac, "class list"))
     return;
 
-  if (buf)
-    xfree (buf);
-  class = buf = strdup(*av);
-
+  searching = 1;
+  search->class = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -931,15 +1100,11 @@ GNATS_qrtr (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs(ac, "quarter"))
     return;
 
-  if (buf)
-    xfree (buf);
-  quarter = buf = strdup(*av);
-
+  searching = 1;
+  search->quarter = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -948,15 +1113,11 @@ GNATS_kywd (ac, av)
      int ac;
      char **av;
 {
-  static char *buf = (char *)NULL;
-
   if (! chk_nargs(ac, "keyword list"))
     return;
 
-  if (buf)
-    xfree (buf);
-  keywords = buf = strdup(*av);
-
+  searching = 1;
+  search->keywords = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -1031,6 +1192,7 @@ GNATS_abfr (ac, av)
     printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
+      searching = 1;
       arrived_before = t;
       printf ("%d Ok.\r\n", CODE_OK);
     }
@@ -1051,6 +1213,7 @@ GNATS_araf (ac, av)
     printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
+      searching = 1;
       arrived_after = t;
       printf ("%d Ok.\r\n", CODE_OK);
     }
@@ -1071,6 +1234,7 @@ GNATS_cbfr (ac, av)
     printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
+      searching = 1;
       closed_before = t;
       printf ("%d Ok.\r\n", CODE_OK);
     }
@@ -1091,6 +1255,7 @@ GNATS_caft (ac, av)
     printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
+      searching = 1;
       closed_after = t;
       printf ("%d Ok.\r\n", CODE_OK);
     }
@@ -1111,6 +1276,7 @@ GNATS_mbfr (ac, av)
     printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
+      searching = 1;
       modified_before = t;
       printf ("%d Ok.\r\n", CODE_OK);
     }
@@ -1131,6 +1297,7 @@ GNATS_maft (ac, av)
     printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
+      searching = 1;
       modified_after = t;
       printf ("%d Ok.\r\n", CODE_OK);
     }
@@ -1201,7 +1368,6 @@ GNATS_mtxt (ac, av)
 {
   if (! chk_nargs (ac, "regular expression"))
     return;
-
   searching = 1;
   m_text_search = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
@@ -1228,6 +1394,10 @@ GNATS_rset (ac, av)
   xfree (search->confidential);
   xfree (search->severity);
   xfree (search->priority);
+  xfree (search->originator);
+  xfree (search->class);
+  xfree (search->release);
+  xfree (search->synopsis);
   memset (search, 0, sizeof (Index));
 
   clean_pr ();
@@ -1235,10 +1405,6 @@ GNATS_rset (ac, av)
   if (check_index ())
     new_index = 1;
 
-  xfree (originator);
-  originator = (char *) NULL;
-  xfree (class);
-  class = (char *) NULL;
   searching = skip_closed = 0;
   xfree (text_search);
   text_search = (char *) NULL;
@@ -1246,15 +1412,15 @@ GNATS_rset (ac, av)
   m_text_search = (char *) NULL;
 #ifdef GNATS_RELEASE_BASED
   required_before = required_after = (time_t)0;
-  xfree (quarter);
-  quarter = (char *) NULL;
-  xfree (keywords);
-  keywords = (char *) NULL;
+  xfree (search->quarter);
+  xfree (search->keywords);
 #endif
   arrived_before  = arrived_after  = (time_t)0;
   modified_before = modified_after = (time_t)0;
   closed_before   = closed_after   = (time_t)0;
 
+  set_confidential_access ();
+
   if (new_index)
     printf ("%d Reset state...reloaded the index.\r\n", CODE_OK);
   else
@@ -1504,6 +1670,7 @@ GNATS_auth (ac, av)
 
       printf ("%d Okie-dokey.\r\n", CODE_OK);
       client_authorized = 1;
+      host_access = user_access = ACCESS_EDIT;
       return;
     }
 #endif
@@ -1668,7 +1835,7 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-   VDAT <date>             validate the format of <date>\r\n",
 	  CODE_INFORMATION);
-  printf ("%d-   SVTY <sevty>            search for PRs which are of the severity <sevty>\r\n",
+  printf ("%d-   SVTY <severity>         search for PRs which are of the severity <severity>\r\n",
 	  CODE_INFORMATION);
   printf ("%d-   STAT <state>            search for PRs in the state <state>\r\n",
 	  CODE_INFORMATION);
@@ -1698,10 +1865,19 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-   EDIT <PR>               check in edited <PR>\r\n",
 	  CODE_INFORMATION);
+  printf ("%d-   CHDB <database>         change GNATS ROOT to <database>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   DBLS                    list databases known to server\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   DBLA                    list database aliases known to server\r\n",
+	  CODE_INFORMATION);
+
 #ifdef HAVE_KERBEROS
   printf ("%d-   AUTH <kerberos>	 Kerberos authentication\r\n",
 	  CODE_INFORMATION);
 #endif
+  printf ("%d-   USER <userid> <passwd>	 UserId and password for database access.\r\n",
+	  CODE_INFORMATION);
   printf ("%d-The search itself is taken by either QURY, SUMM, or FULL.  Each\r\n",
 	  CODE_INFORMATION);
   printf ("%d-  may be invoked with no arguments, which will search the entire\r\n",
@@ -1716,9 +1892,9 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-  FULL command will show the entire header and body of the PR.\r\n",
 	  CODE_INFORMATION);
-  printf ("%d-You may also use the option SQLF, which will give you an SQL format\r\n",
+  printf ("%d-You may also use the option SQLF or SQL2, which will give you an SQL\r\n",
 	  CODE_INFORMATION);
-  printf ("%d-  line for each PR, which you can bring into your own database.\r\n",
+  printf ("%d-  format line for each PR, which you can bring into your own database.\r\n",
 	  CODE_INFORMATION);
   printf ("%d End of HELP info for GNATS daemon.\r\n", CODE_INFORMATION);
 }
