@@ -21,7 +21,7 @@
 
 debug_print=false # or echo to get output.
 DATE=`LC_TIME=C date`
-GNATS_ROOT=xGNATS_ROOTx
+GNATS_ROOT=${GNATS_ROOT:-xGNATS_ROOTx}; export GNATS_ROOT
 LIBEXECDIR=xLIBEXECDIRx
 PATH=$PATH:$GNATS_ROOT/gnats-bin
 PR_EDIT=$LIBEXECDIR/gnats/pr-edit
@@ -29,7 +29,7 @@ PR_ADDR=$LIBEXECDIR/gnats/pr-addr
 locked=
 version=xVERSIONx
 
-usage="Usage: $0 [-hV] [--version] [--help] PR"
+usage="Usage: $0 [-Vh] [--version] [--help] PR"
 
 # Newer config information?
 [ -f ${GNATS_ROOT}/gnats-adm/config ] && . ${GNATS_ROOT}/gnats-adm/config
@@ -54,7 +54,7 @@ if [ $# -eq 0 ]; then
   echo "$usage" ; exit 1
 fi
 case "$1" in
-  -V|-v|--version|--ve*)
+  -V|--version|--ve*)
     echo "$version"; exit 0
     ;;
   -h|--help*)
@@ -67,7 +67,7 @@ case "$1" in
     if [ "`echo $1 | grep /`" != "" ]; then
       full_id=$1
     else
-      full_id=`grep "/$1:" $GNATS_ROOT/gnats-adm/index | awk -F: '{print $1}' -`
+      full_id=`grep "/$1|" $GNATS_ROOT/gnats-adm/index | awk -F"|" '{print $1}' -`
     fi
     ;;
 esac
@@ -155,7 +155,7 @@ rm -f $new.old
 # call $VISUAL or exit
 checking=t
 while [ "$checking" != "" ]; do
-  errors="`$PR_EDIT --check < $new`"
+  errors="`$PR_EDIT --check < $new 2>&1`"
   if [ "$errors" != "" ]; then
     echo "Hit \`return\` to fix the following errors, or type \'quit\' to quit:"
     echo "$errors"
