@@ -574,13 +574,13 @@ reply_to_submitter (responsible)
   if (*reply_to == '\0')
     reply_to = header_value (FROM);
   fprintf (msg, "To: %s", reply_to);
-  fprintf (msg, "From: %s\n", gnats_addr);
+  fprintf (msg, "From: %s\n", gnats_admin);
   subject = header_value (SUBJECT);
   fprintf (msg, "Subject: Re: %s/%s: %s",
 	   field_value (CATEGORY), field_value (NUMBER), subject);
   if (*subject == '\0')
     fprintf (msg, "\n");
-  fprintf (msg, "Reply-To: %s, %s\n", gnats_addr, responsible);
+  fprintf (msg, "Reply-To: %s, %s\n", gnats_admin, responsible);
   fprintf (msg, "In-Reply-To: Your message of %s\t%s\n",
 	   header_value (DATE), header_value (MSG_ID));
 
@@ -730,7 +730,7 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
 	     header_value (DATE), header_value (MSG_ID));
 
 #else /* ! DONT_USE_RESENT */
-  fprintf(msg, "Resent-From: %s (GNATS Management)\n", gnats_user);
+  fprintf(msg, "Resent-From: %s (GNATS Management)\n", gnats_admin);
   fprintf(msg, "Resent-To: %s\n", responsible);
   if (*notify)
     fprintf(msg, "Resent-Cc: %s\n", notify);
@@ -738,7 +738,7 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
     char * reply_to = header_value (REPLY_TO);
     if (! (reply_to && *reply_to))
       reply_to = header_value (FROM);
-    fprintf(msg, "Resent-Reply-To: %s, %s", gnats_addr, reply_to);
+    fprintf(msg, "Resent-Reply-To: %s, %s", gnats_admin, reply_to);
   }
   for (i = RETURN_PATH; i < NUM_HEADER_ITEMS; i++)
     {
@@ -843,7 +843,7 @@ append_report (infile, filename)
      char *filename;
 {
   FILE *pr_file, *msg;
-  char *line = (char *) xmalloc (STR_MAX);
+  char *line = (char *) xmalloc (STR_MAXLONG);
   char *buf, *b;
   int buf_len, buf_max, l;
   Responsible *responsible;
@@ -852,6 +852,9 @@ append_report (infile, filename)
   char *number;
   char *from, *to, *cc = " \n", *subject = " \n", *date;
   time_t t;
+  /* Where to keep the static index if necessary.  */
+  Index *current_index = (Index *)NULL;
+  Index *i, *prev_index = NULL, *old_index = NULL;
 
   /* Save the values we read while in main().  */
   from = (char *) strdup (header_value (FROM));
@@ -900,7 +903,7 @@ append_report (infile, filename)
   APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
   APPEND_STRING ((char*)"\n", buf, b, buf_len, buf_max, l, 0);
 
-  while ((fgets (line, STR_MAX, infile)) != NULL)
+  while ((fgets (line, STR_MAXLONG, infile)) != NULL)
     {
       /* Quote all correspondence so PR doesn't get messed up */
       APPEND_STRING ((char*)" ", buf, b, buf_len, buf_max, l, 0);
@@ -931,6 +934,29 @@ append_report (infile, filename)
   /* Now write out the PR, with that new field.  */
   create_report (filename, 1);
 
+  number = pr_number_from_path (filename);
+
+  /* Now update the index with the new Last-Modified date.  */
+  current_index = get_index ();
+  for (i = current_index; i ; i = i->next)
+    {
+      /* Let's see if we can find what we are looking for.  */
+      if (strcmp (i->number, number) == 0)
+        {
+          t = get_date (date, NULL);         
+          i->last_modified = (char *) xmalloc (18);
+          sprintf (i->last_modified, "%d", t);    
+          /* write out the new index.  */
+          write_index (current_index);
+          break; 
+        }
+    }
+  free_index (current_index);
+  if (i == NULL)
+    {
+      punt (1, "%s: can't find index entry for %s\n", program_name, number);
+    }
+
   /* If it's "foo (Foo Bar)", chop off the rest.  */
   p = field_value (RESPONSIBLE);
   q = (char *) strchr (p, ' ');
@@ -947,8 +973,6 @@ append_report (infile, filename)
     punt (1, "%s: cannot open mail agent.", program_name);
   fprintf (msg, "To: %s\n", responsible->alias);
 
-  number = pr_number_from_path (filename);
-
   if (find_submitter (&submitter, field_value (SUBMITTER)) == -1)
     punt (0, "%s: PR %s has a Submitter field of `%s', which is invalid.",
 	  program_name, number, field_value (SUBMITTER));
