@@ -87,7 +87,7 @@ do_category (c)
   /* Allocate for a PR with 8 digits.  */
   char *path = (char *) xmalloc (len + 9);
   char *p;
-  char line[STR_MAX];
+  char *line = (char *) xmalloc (STR_MAXLONG);
 
   errno = 0;
   if (chdir (gnats_root) < 0)
@@ -98,7 +98,11 @@ do_category (c)
     }
 
   if (c == NULL || *c == '\0')
-    return;
+    {
+      xfree (line);
+      xfree (path);
+      return;
+    }
 
   errno = 0;
   d = opendir (c);
@@ -106,6 +110,7 @@ do_category (c)
     {
       fprintf (stderr, "%s: can't open the category directory %s/%s: %s\n",
 	       program_name, gnats_root, c, strerror (errno));
+      xfree (line);
       xfree (path);
       return;
     }
@@ -162,6 +167,7 @@ do_category (c)
       }
 
   closedir (d);
+  xfree (line);
   xfree (path);
 }
 
@@ -332,6 +338,7 @@ main (argc, argv)
 
   configure ();
   init_gnats ();
+  local_chdb(&gnats_root, program_name);
   umask (022);
 
   if (sort_numerical)
@@ -350,6 +357,7 @@ main (argc, argv)
     }
 
   fclose (outfile);
+  free_db_conf();
   exit (0);
 }
 
