@@ -201,6 +201,8 @@ extern void get_reply			PARAMS((void));
 extern void client_init			PARAMS((void));
 extern void client_exit			PARAMS((void));
 extern void safe_exit			PARAMS((void));
+extern void client_chdb 	        PARAMS((char *));
+extern void client_user 	        PARAMS((char *, char *));
 
 /* in config.c */
 extern void configure			PARAMS((void));
@@ -268,6 +270,7 @@ extern Responsible *get_responsible_address	PARAMS((char *));
 /* in pr.c */
 extern void read_pr			PARAMS((FILE *, int));
 extern void write_pr			PARAMS((FILE *, PR_Name));
+extern void write_pr_from_index		PARAMS((FILE *, PR_Name, char *));
 extern void init_pr			PARAMS((void));
 extern void clean_pr			PARAMS((void));
 extern char *field_value		PARAMS((PR_Name));
