@@ -36,6 +36,9 @@ typedef enum _msg_type {
 
 /* in gnatsd.c */
 extern MsgType get_line			PARAMS((char *, int, int));
+extern int     allowed                  PARAMS((char *, char *, char *, int *));
+extern int     get_user_access          PARAMS((char *, char *, char *, char *));
+extern void    set_confidential_access  PARAMS((void));
 
 /* cmds.c */
 extern void GNATS_helo			PARAMS((int, char**));
@@ -43,11 +46,15 @@ extern void GNATS_quit			PARAMS((int, char**));
 extern void GNATS_qury			PARAMS((int, char**));
 extern void GNATS_summ			PARAMS((int, char**));
 extern void GNATS_sqlf			PARAMS((int, char**));
+extern void GNATS_sql2			PARAMS((int, char**));
 extern void GNATS_full			PARAMS((int, char**));
 extern void GNATS_lock			PARAMS((int, char**));
 extern void GNATS_unlk			PARAMS((int, char**));
 extern void GNATS_lkdb			PARAMS((int, char**));
 extern void GNATS_undb			PARAMS((int, char**));
+extern void GNATS_chdb			PARAMS((int, char**));
+extern void GNATS_dbls			PARAMS((int, char**));
+extern void GNATS_dbla			PARAMS((int, char**));
 extern void GNATS_edit			PARAMS((int, char**));
 extern void GNATS_mlpr			PARAMS((int, char**));
 extern void GNATS_mlct			PARAMS((int, char**));
@@ -82,6 +89,7 @@ extern void GNATS_text			PARAMS((int, char**));
 extern void GNATS_mtxt			PARAMS((int, char**));
 extern void GNATS_rset			PARAMS((int, char**));
 extern void GNATS_auth			PARAMS((int, char**));
+extern void GNATS_user			PARAMS((int, char**));
 extern void GNATS_lcat			PARAMS((int, char**));
 extern void GNATS_lcfg			PARAMS((int, char**));
 extern void GNATS_lcla			PARAMS((int, char**));
@@ -121,6 +129,17 @@ extern void GNATS_kywd			PARAMS((int, char**));
 #define REPLY_CONT	1
 #define REPLY_END	2
 
+/* gnatsd flags, including host and user access levels */
+#define FLAG_MASK	0xF00
+#define FLAG_ONE_ARG	0x100
+#define ACCESS_MASK     0xFF
+#define ACCESS_DENY     0x01
+#define ACCESS_NONE     0x02
+#define ACCESS_VIEW     0x03
+#define ACCESS_VIEWCONF 0x04
+#define ACCESS_EDIT     0x05
+#define ACCESS_ADMIN    0x06
+
 /* Used if Kerberos version 4 authentication is enabled.  */
 #define GNATS_KRB4_VERSIONID		"GnatsK1.0"
 #define GNATS_KRB4_PRINCIPAL_NAME	gnats_user
