@@ -73,6 +73,8 @@ struct option long_options[] =
   {"outfile", 1, NULL, 'o'},
   {"host", 1, NULL, 'H'},
   {"port", 1, NULL, 'P'},
+  {"user", 1, NULL, 'v'},
+  {"passwd", 1, NULL, 'w'},
   {"debug", 0, NULL, 'D'},
   {"version", 0, NULL, 'V'},
   {"help", 0, NULL, 'h'},
@@ -154,6 +156,14 @@ net_check_pr (fp)
   if (pr[DATE_REQUIRED].value && pr[DATE_REQUIRED].value[0])
     do_field (pr[DATE_REQUIRED].value, "VDAT");
 #endif
+
+  if (pr[ARRIVAL_DATE].value && pr[ARRIVAL_DATE].value[0])
+    do_field (pr[ARRIVAL_DATE].value, "VDAT");
+  if (pr[LAST_MODIFIED].value && pr[LAST_MODIFIED].value[0])
+    do_field (pr[LAST_MODIFIED].value, "VDAT");
+  if (pr[CLOSED_DATE].value && pr[CLOSED_DATE].value[0])
+    do_field (pr[CLOSED_DATE].value, "VDAT");
+  
   finish_field_validations ();
 
 #if 0
@@ -234,12 +244,13 @@ main (argc, argv)
   char *username;
   int result = 1, get_lists = 0;
   char *processid = (char *)NULL;
+  char *user = "anonymous", *passwd = "guest";
 
   program_name = basename (argv[0]);
   host_name = gnats_server;
   edit_options = MODIFY;
 
-  while ((option = getopt_long (argc, argv, "l:uLUcgd:f:o:p:H:DVhP:",
+  while ((option = getopt_long (argc, argv, "l:uLUcgd:f:o:p:H:v:w:DVhP:",
 				long_options, (int *)0)) != EOF)
     {
       switch (option)
@@ -308,6 +319,14 @@ main (argc, argv)
 	  port = atoi (optarg);
 	  break;
 
+        case 'v':
+          user = optarg;
+          break;
+
+        case 'w':
+          passwd = optarg;
+          break;
+
 	case 'D':
 	  debug = 1;
 	  break;
@@ -347,6 +366,10 @@ main (argc, argv)
 
   client_init ();
 
+  client_chdb (gnats_root);
+
+  client_user (user, passwd);
+
   switch (edit_options) {
     case LOCK:
       if (debug)
@@ -410,9 +433,10 @@ usage ()
 {
   fprintf (stderr, "\
 Usage: %s [-cuLUVFh] [-d directory] [-f filename] [-l username] [-o filename]\n\
-          [--directory=directory] [--filename=filename]\n\
+          [-v userid] [-w password] [--directory=directory] [--filename=filename]\n\
           [--lock=user] [--process=pid] [--unlock] [--lockdb] [--unlockdb]\n\
 	  [--host=server] [--outfile=filename] [--help] [--version]\n\
+          [--user=userid] [--passwd=password]\n\
           [--check] [PR]\n", program_name);
 }
 
