@@ -116,6 +116,8 @@ struct option long_options[] =
   {"help", 0, NULL, 'h'},
   {"host", 1, NULL, 'H'},
   {"port", 1, NULL, PORT_OPTION},
+  {"user", 1, NULL, 'v'},
+  {"passwd", 1, NULL, 'w'},
   {"multitext", 1, NULL, 'm'},
   {"originator", 1, NULL, 'O'},
   {"release", 1, NULL, 'A'},
@@ -132,6 +134,7 @@ struct option long_options[] =
   {"severity", 1, NULL, 'e'},
   {"skip-closed", 0, NULL, 'x'},
   {"sql", 0, NULL, 'i'},
+  {"sql2", 0, NULL, 'I'},
   {"state", 1, NULL, 's'},
   {"summary", 0, NULL, 'q'},
   {"submitter", 1, NULL, 'S'},
@@ -282,53 +285,53 @@ ask_about (info, args)
       get_reply ();
     }
 
-  if (originator)
+  if (info->originator)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `ORIG %s'\n", program_name, originator);
-      fprintf (serv_write, "ORIG %s\r\n", originator);
+	fprintf (stderr, "%s: writing `ORIG %s'\n", program_name, info->originator);
+      fprintf (serv_write, "ORIG %s\r\n", info->originator);
       get_reply ();
     }
 
-  if (class)
+  if (info->class)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `CLSS %s'\n", program_name, class);
-      fprintf (serv_write, "CLSS %s\r\n", class);
+	fprintf (stderr, "%s: writing `CLSS %s'\n", program_name, info->class);
+      fprintf (serv_write, "CLSS %s\r\n", info->class);
       get_reply ();
     }
 
 #ifdef GNATS_RELEASE_BASED
-  if (quarter)
+  if (info->quarter)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `QRTR %s'\n", program_name, quarter);
-      fprintf (serv_write, "QRTR %s\r\n", quarter);
+	fprintf (stderr, "%s: writing `QRTR %s'\n", program_name, info->quarter);
+      fprintf (serv_write, "QRTR %s\r\n", info->quarter);
       get_reply ();
     }
 
-  if (keywords)
+  if (info->keywords)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `KYWD %s'\n", program_name, keywords);
-      fprintf (serv_write, "KYWD %s\r\n", keywords);
+	fprintf (stderr, "%s: writing `KYWD %s'\n", program_name, info->keywords);
+      fprintf (serv_write, "KYWD %s\r\n", info->keywords);
       get_reply ();
     }
 #endif
 
-  if (synopsis)
+  if (info->synopsis)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `SYNP %s'\n", program_name, synopsis);
-      fprintf (serv_write, "SYNP %s\r\n", synopsis);
+	fprintf (stderr, "%s: writing `SYNP %s'\n", program_name, info->synopsis);
+      fprintf (serv_write, "SYNP %s\r\n", info->synopsis);
       get_reply ();
     }
 
-  if (release)
+  if (info->release)
     {
       if (debug)
-	fprintf (stderr, "%s: writing `RLSE %s'\n", program_name, release);
-      fprintf (serv_write, "RLSE %s\r\n", release);
+	fprintf (stderr, "%s: writing `RLSE %s'\n", program_name, info->release);
+      fprintf (serv_write, "RLSE %s\r\n", info->release);
       get_reply ();
     }
 
@@ -429,6 +432,8 @@ ask_about (info, args)
     cmd = "FULL";
   else if (query_format & FORMAT_SQL)
     cmd = "SQLF";
+  else if (query_format & FORMAT_SQL2)
+    cmd = "SQL2";
   else
     cmd = "QURY";
 
@@ -463,6 +468,7 @@ main (argc, argv)
   int optc;
   Index *info;
   int formats = 0, lists = 0;
+  char *user = "anonymous", *passwd = "guest";
 
   program_name = basename (argv[0]);
   if (argc == 1)
@@ -474,22 +480,16 @@ main (argc, argv)
   bzero ((Index *) info, sizeof (Index));
 
 #ifdef GNATS_RELEASE_BASED
-  while ((optc = getopt_long (argc, argv, "a:A:b:B:c:C:Dd:e:K:L:m:M:o:O:p:PQ:s:S:r:t:u:U:y:z:Z:VFGixhqH:RjJklT",
+  while ((optc = getopt_long (argc, argv, "a:A:b:B:c:C:Dd:e:K:L:m:M:o:O:p:PQ:s:S:r:t:u:U:v:w:y:z:Z:VFGiIxhqH:RjJklT",
 #else
-  while ((optc = getopt_long (argc, argv, "a:A:b:B:c:C:Dd:e:L:m:M:o:O:p:Ps:S:r:t:y:z:Z:VFGixhqH:RjJklT",
+  while ((optc = getopt_long (argc, argv, "a:A:b:B:c:C:Dd:e:L:m:M:o:O:p:Ps:S:r:t:v:w:y:z:Z:VFGiIxhqH:RjJklT",
 #endif
 			      long_options, (int *) 0)) != EOF)
     {
       switch (optc)
 	{
 	case 'd':
-#if 0
-	  spec_gnats_root = optarg;
-#else
-	  fprintf (stderr, "%s: directory option not valid for network query-pr\n",
-		   program_name);
-	  exit (1);
-#endif
+	  gnats_root = optarg;
 	  break;
 
 	case 'D':
@@ -497,7 +497,7 @@ main (argc, argv)
 	  break;
 
 	case 'A':
-	  release = optarg;
+	  info->release = optarg;
 	  break;
 
 	case 'a':
@@ -562,16 +562,16 @@ main (argc, argv)
 	  break;
 
 	case 'L':
-	  class = optarg;
+	  info->class = optarg;
 	  break;
 
 #ifdef GNATS_RELEASE_BASED
 	case 'K':
-	  keywords = optarg;
+	  info->keywords = optarg;
 	  break;
 
 	case 'Q':
-	  quarter = optarg;
+	  info->quarter = optarg;
 	  break;
 #endif
 
@@ -580,7 +580,7 @@ main (argc, argv)
 	  break;
 
 	case 'O':
-	  originator = optarg;
+	  info->originator = optarg;
 	  break;
 
 	case 'p':
@@ -621,6 +621,11 @@ main (argc, argv)
 	  formats++;
 	  break;
 
+	case 'I':
+	  query_format |= FORMAT_SQL2;
+	  formats++;
+	  break;
+
 	case 'q':
 	  query_format |= FORMAT_SUMM;
 	  formats++;
@@ -671,7 +676,7 @@ main (argc, argv)
 #endif
 
 	case 'y':
-	  synopsis = optarg;
+	  info->synopsis = optarg;
 	  break;
 
 	case 'H':
@@ -682,6 +687,14 @@ main (argc, argv)
 	  port = atoi (optarg);
 	  break;
 
+        case 'v':
+          user = optarg;
+          break;
+
+        case 'w':
+          passwd = optarg;
+          break;
+
 	case 'h':
 	  usage (0);
 	  break;
@@ -707,6 +720,10 @@ main (argc, argv)
 
   client_init ();
 
+  client_chdb(gnats_root);
+
+  client_user(user, passwd);
+
   /* Now make our query, and act on what they said.  */
   ask_about (info, argv + optind);
   get_reply ();
@@ -722,13 +739,13 @@ usage (s)
 {
 #ifdef GNATS_RELEASE_BASED
   fprintf (stderr, "\
-Usage: %s [-DFGHhiPqVxE] [-C confidential] [-c category] [-d directory]\n\
+Usage: %s [-DFGHhiIqVxE] [-C confidential] [-c category] [-d directory]\n\
        [-e severity] [-m mtext] [-O originator] [-o outfile] [-p priority]\n\
        [-L class] [-r responsible] [-S submitter] [-s state] [-t text]\n\
        [-b date] [-a date] [-B date] [-M date] [-z date] [-Z date]\n\
-       [-y synopsis] [-A release] [-H host] [-P port] [--full] [--help]\n\
-       [--print-path] [--version] [--summary] [--sql]\n\
-       [--skip-closed] [--category=category]\n\
+       [-y synopsis] [-A release] [-H host] [-P port] [-v userid] [-w password]\n\
+       [--full] [--help][--print-path] [--version] [--summary]\n\
+       [--sql] [--sql2][--skip-closed] [--category=category]\n\
        [--confidential=yes|no] [--directory=directory] [--output=outfile]\n\
        [--originator=name] [--priority=level] [--class=class]\n\
        [--severity=severity] [--state=state] [--submitter=submitter]\n\
@@ -740,17 +757,18 @@ Usage: %s [-DFGHhiPqVxE] [-C confidential] [-c category] [-d directory]\n\
        [--closed-before=date] [--closed-after=date]\n\
        [--list-categories] [--list-classes] [--list-responsible]\n\
        [--list-states] [--list-submitters] [--list-config]\n\
-       [--text=text] [--multitext=mtext] [--host=server] [--port=port] [PR] [PR]...\n",
+       [--text=text] [--multitext=mtext] [--host=server] [--port=port]\n\
+       [--user=userid] [--passwd=password] [PR] [PR]...\n",
 	   program_name);
 #else
   fprintf (stderr, "\
-Usage: %s [-DFGHhiPqVxE] [-C confidential] [-c category] [-d directory]\n\
+Usage: %s [-DFGHhiIPqVxE] [-C confidential] [-c category] [-d directory]\n\
        [-e severity] [-m mtext] [-O originator] [-o outfile] [-p priority]\n\
        [-L class] [-r responsible] [-S submitter] [-s state] [-t text]\n\
        [-b date] [-a date] [-B date] [-M date] [-z date] [-Z date]\n\
-       [-y synopsis] [-A release] [-H host] [-P port] [--full] [--help]\n\
-       [--print-path] [--version] [--summary] [--sql]\n\
-       [--skip-closed] [--category=category]\n\
+       [-y synopsis] [-A release] [-H host] [-P port] [-v userid] [-w password]\n\
+       [--full] [--help] [--print-path] [--version] [--summary]\n\
+       [--sql] [--sql2] [--skip-closed] [--category=category]\n\
        [--confidential=yes|no] [--directory=directory] [--output=outfile]\n\
        [--originator=name] [--priority=level] [--class=class]\n\
        [--severity=severity] [--state=state] [--submitter=submitter]\n\
