@@ -278,6 +278,33 @@ write_pr (fp, string)
   	  fprintf (fp, "%s%s", pr[i].name, ret);
     }
 }
+
+/* Write either a single PR field from the index */
+void 
+write_pr_from_index (fp, name, value)
+     FILE *fp;
+     PR_Name name;
+     char *value;
+{
+  extern char *ret;
+  char   fmt[10], *t;
+  time_t time;
+
+  sprintf (fmt, "%%-16s %%s%s", ret);
+
+  if (pr[name].datatype == Date)
+    {
+      t = (char *) xmalloc (GNATS_TIME_LENGTH);
+      if ((time = (time_t) atol (value)) == 0)
+        t[0] = '\0';
+      else
+        strftime (t, GNATS_TIME_LENGTH, "%a %b %e %T %Z %Y", localtime (&time));
+      fprintf (fp, fmt, pr[name].name, t);
+      xfree (t);
+    }
+  else
+    fprintf (fp, fmt, pr[name].name, value);
+}
 
 
 /* ****** OBSOLETE ******* Replaced by init_states() and init_classes() in files.c */
@@ -534,21 +561,19 @@ init_pr ()
 
   pr[KEYWORDS].name = KEYWORDS_STRING;
   pr[KEYWORDS].datatype = Text;
+
+  pr[DATE_REQUIRED].name = DATE_REQUIRED_STRING;
+  pr[DATE_REQUIRED].datatype = Date;
 #endif
 
   pr[RELEASE].name = RELEASE_STRING;
   pr[RELEASE].datatype = Text;
 
   pr[LAST_MODIFIED].name = LAST_MODIFIED_STRING;
-  pr[LAST_MODIFIED].datatype = Text;
+  pr[LAST_MODIFIED].datatype = Date;
 
   pr[CLOSED_DATE].name = CLOSED_DATE_STRING;
-  pr[CLOSED_DATE].datatype = Text;
-
-#ifdef GNATS_RELEASE_BASED
-  pr[DATE_REQUIRED].name = DATE_REQUIRED_STRING;
-  pr[DATE_REQUIRED].datatype = Date;
-#endif
+  pr[CLOSED_DATE].datatype = Date;
 
   pr[ENVIRONMENT].name = ENVIRONMENT_STRING;
   pr[ENVIRONMENT].datatype = MultiText;
