@@ -111,14 +111,32 @@ sql_time (s)
 {
   time_t t;
   struct tm *ar_time;
-  /* Note: we deliberately use 18 here, since that is what the width of
+  /* Note: we deliberately use 21 here, since that is what the width of
      this time string will end up looking like.  In the case where we
      use things relative to timezone and such, it'd grow---but not here.  */
-  char *buf = (char *) xmalloc (18);
+  char *buf = (char *) xmalloc (21);
 
   t = get_date (s, NULL);
   ar_time = (struct tm *) localtime (&t);
-  strftime (buf, 18, "%Y-%m-%d %H:%M", ar_time);
+  strftime (buf, 21, "%Y-%m-%d %H:%M:%S", ar_time);
+
+  return buf;
+}
+
+char *
+sql_time_from_index (s)
+     char *s;
+{
+  time_t t;
+  struct tm *ar_time;
+  /* Note: we deliberately use 21 here, since that is what the width of
+     this time string will end up looking like.  In the case where we
+     use things relative to timezone and such, it'd grow---but not here.  */
+  char *buf = (char *) xmalloc (21);
+
+  t = (time_t) atol (s);
+  ar_time = (struct tm *) localtime (&t);
+  strftime (buf, 21, "%Y-%m-%d %H:%M:%S", ar_time);
 
   return buf;
 }
@@ -185,8 +203,8 @@ regcmp (pat, match)
   switch (r.i)
     {
     case -2:
-      fprintf (stderr, "%s: warning: re_match died on %s\n",
-	       program_name, pr[NUMBER].value);
+      fprintf (stderr, "%s: warning: re_match died with pattern %s and string $s\n",
+	       program_name, pat, match);
       /*FALLTHRU*/
     case -1:
       return 1;
@@ -233,8 +251,8 @@ regfind (pat, match)
   switch (r.i)
     {
     case -2:
-      fprintf (stderr, "%s: warning: re_search died on %s\n",
-	       program_name, pr[NUMBER].value);
+      fprintf (stderr, "%s: warning: re_match died with pattern %s and string $s\n",
+	       program_name, pat, match);
       /*FALLTHRU*/
     case -1:
       return 1;
@@ -313,45 +331,98 @@ date_compare (tst, sdate, type, convert_p)
 }
 
 int
-pr_matches (s, i)
+pr_matches (s, i, opened)
      Index *s;
      Index *i;
+     int   *opened;
 {
+  int status;
+  
+  *opened = 0;
+
   if (skip_closed && check_state_type (i->state, "closed"))
     return 0;
-  if (!s || !searching) return 1;
-  return (!s->category || (regcmp (s->category, i->category) == 0))
-    && (!s->submitter || (regcmp (s->submitter, i->submitter) == 0))
-    && (!s->responsible || (regcmp (s->responsible, i->responsible) == 0))
-    && (!s->state || (regcmp (s->state, i->state) == 0))
-    && (!s->confidential || (regcmp (s->confidential, i->confidential) == 0))
+
+  if (!s || !searching)
+    return 1;
+
+  if (!(
+       (!s->category     || (regcmp  (s->category,     i->category)     == 0))
+    && (!s->submitter    || (regcmp  (s->submitter,    i->submitter)    == 0))
+    && (!s->responsible  || (regcmp  (s->responsible,  i->responsible)  == 0))
+    && (!s->state        || (regcmp  (s->state,        i->state)        == 0))
+    && (!s->confidential || (regcmp  (s->confidential, i->confidential) == 0))
+    && (!s->severity     || (regcmp  (s->severity,     i->severity)     == 0))
+    && (!s->priority     || (regcmp  (s->priority,     i->priority)     == 0))
+    && (!s->originator   || (regfind (s->originator,   i->originator)   == 0))
+    && (!s->class        || (regfind (s->class,        i->class)        == 0))
+    && (!s->synopsis     || (regfind (s->synopsis,     i->synopsis)     == 0))
+    && (!s->release      || (regfind (s->release,      i->release)      == 0))
 #ifdef GNATS_RELEASE_BASED
-    /* We can check this here cuz the date required is included in the index.  */
+    && (!s->quarter      || (regfind (s->quarter,      i->quarter)      == 0))
+    && (!s->keywords     || (regfind (s->keywords,     i->keywords)     == 0))
     && (!required_before || (date_compare (required_before, i->date_required, 0, NOCONVERT) == 0))
-    && (!required_after || (date_compare (required_after, i->date_required, 1, NOCONVERT) == 0))
+    && (!required_after  || (date_compare (required_after,  i->date_required, 1, NOCONVERT) == 0))
 #endif
-    && (!s->severity || (regcmp (s->severity, i->severity) == 0))
-    && (!s->priority || (regcmp (s->priority, i->priority) == 0));
+    && (!arrived_before  || (date_compare (arrived_before,  i->arrival_date,  0, NOCONVERT) == 0))
+    && (!arrived_after   || (date_compare (arrived_after,   i->arrival_date,  1, NOCONVERT) == 0))
+    && (!modified_before || (date_compare (modified_before, i->last_modified, 0, NOCONVERT) == 0))
+    && (!modified_after  || (date_compare (modified_after,  i->last_modified, 1, NOCONVERT) == 0))
+    && (!closed_before   || (date_compare (closed_before,   i->closed_date,   0, NOCONVERT) == 0))
+    && (!closed_after    || (date_compare (closed_after,    i->closed_date,   1, NOCONVERT) == 0))
+    && (!text_search     || (check_text (s, i) == 1))))
+       return 0;
+
+  if (m_text_search)
+    {
+      int    val = 0;
+      char *path = make_path (i->category, i->number);
+      
+      if (val = get_pr (path, i->number, quiet))
+         *opened = 1;
+      xfree (path);
+
+      if (!val || !check_mtext())
+        return 0;
+    }
+  
+  return 1;
 }
 
 int
-check_text ()
+check_text (s, i)
+     Index *s;
+     Index *i;
+{
+  if (text_search)
+    {
+  return (
+       /* Note that text_search uses regfind, but the individual query options use regcmp */
+       (regfind (text_search, i->category)     == 0)
+    || (regfind (text_search, i->submitter)    == 0)
+    || (regfind (text_search, i->responsible)  == 0)
+    || (regfind (text_search, i->state)        == 0)
+    || (regfind (text_search, i->confidential) == 0)
+    || (regfind (text_search, i->severity)     == 0)
+    || (regfind (text_search, i->priority)     == 0)
+    || (regfind (text_search, i->originator)   == 0)
+    || (regfind (text_search, i->class)        == 0)
+    || (regfind (text_search, i->synopsis)     == 0)
+    || (regfind (text_search, i->release)      == 0)
+#ifdef GNATS_RELEASE_BASED
+    || (regfind (text_search, i->quarter)      == 0)
+    || (regfind (text_search, i->keywords)     == 0)
+#endif
+       );
+    }
+  return 1;
+}
+
+int
+check_mtext ()
 {
   PR_Name i;
 
-  if (text_search)
-    {
-      for (i = NUMBER; i < NUM_PR_ITEMS; i++)
-	{
-	  if (pr[i].datatype != Text)
-	    continue;
-	  
-	  if (pr[i].value && regfind (text_search, pr[i].value) == 0)
-	    goto part2;
-	}
-      return 0;
-    }
- part2:
   if (m_text_search)
     {
       for (i = NUMBER; i < NUM_PR_ITEMS; i++)
@@ -376,6 +447,7 @@ get_pr (path, p, silent)
      int silent;
 {
   FILE *fp = fopen (path, "r");
+     
   if (fp == (FILE *)NULL)
     {
       if (! silent)
@@ -397,80 +469,15 @@ get_pr (path, p, silent)
   return 1;
 }
 
-/* Possibly print out a PR, checking if we want to look at unindexed things
-   first.  */
 int
-do_pr_internal (path, prnum)
-     char *path, *prnum;
-{
-  if (originator || text_search || m_text_search
-      || synopsis || release || arrived_before || arrived_after
-      || modified_before || modified_after || closed_before || closed_after
-#ifdef GNATS_RELEASE_BASED
-      || keywords || quarter
-#endif
-      || class)
-    {
-      if (! get_pr (path, prnum, quiet))
-	return 0;
-
-      if ((originator && (pr[ORIGINATOR].value == NULL
-			  || regfind (originator, pr[ORIGINATOR].value) != 0))
-	  || (class && (pr[CLASS].value == NULL
-			|| regfind (class, pr[CLASS].value) != 0))
-	  || (synopsis && (pr[SYNOPSIS].value == NULL
-			|| regfind (synopsis, pr[SYNOPSIS].value) != 0))
-	  || (release && (pr[RELEASE].value == NULL
-			|| regfind (release, pr[RELEASE].value) != 0))
-	  /* We have to check these here cuz the arrival date isn't in the
-	     index, though it probably SHOULD be.  FIXME  */
-	  || (arrived_before  && (pr[ARRIVAL_DATE].value == NULL
-              || *pr[ARRIVAL_DATE].value == 0
-	      || date_compare (arrived_before, pr[ARRIVAL_DATE].value,
-				1, DOCONVERT) == 0))
-	  || (arrived_after   && (pr[ARRIVAL_DATE].value == NULL
-              || *pr[ARRIVAL_DATE].value == 0
-	      || date_compare (arrived_after, pr[ARRIVAL_DATE].value,
-				0, DOCONVERT) == 0))
-	  || (modified_before && (pr[LAST_MODIFIED].value == NULL
-              || *pr[LAST_MODIFIED].value == 0
-	      || date_compare (modified_before, pr[LAST_MODIFIED].value,
-				1, DOCONVERT) == 0))
-	  || (modified_after  && (pr[LAST_MODIFIED].value == NULL
-              || *pr[LAST_MODIFIED].value == 0
-	      || date_compare (modified_after, pr[LAST_MODIFIED].value,
-				0, DOCONVERT) == 0))
-	  || (closed_before   && (pr[CLOSED_DATE].value == NULL
-              || *pr[CLOSED_DATE].value == 0
-	      || date_compare (closed_before, pr[CLOSED_DATE].value,
-				1, DOCONVERT) == 0))
-	  || (closed_after    && (pr[CLOSED_DATE].value == NULL
-              || *pr[CLOSED_DATE].value == 0
-	      || date_compare (closed_after, pr[CLOSED_DATE].value,
-				0, DOCONVERT) == 0))
-#ifdef GNATS_RELEASE_BASED
-	  || (quarter && (pr[QUARTER].value == NULL
-			|| regfind (quarter, pr[QUARTER].value) != 0))
-	  || (keywords && (pr[KEYWORDS].value == NULL
-			|| regfind (keywords, pr[KEYWORDS].value) != 0))
-#endif
-	  || !check_text())
-	return 0;
-      return 1;
-    }
-  else
-    return get_pr (path, prnum, quiet);
-}
-
-int
-do_pr (i)
+do_pr (i, opened)
      Index *i;
+     int   opened;
 {
   int val = 0;
   char *path = make_path (i->category, i->number);
 
-  if ((val = do_pr_internal (path, i->number)))
-    print_pr (path, i->number, 1);
+  val = print_pr (path, i->number, opened, i);
   
   xfree (path);
 
@@ -486,15 +493,16 @@ reset_started ()
   started = 0;
 }
 
-void
-print_pr (path, p, opened)
+int
+print_pr (path, p, opened, i)
      char *path, *p;
      int opened;
+     Index *i;
 {
   /* Note: don't use P in any real way, cuz it could be "1234" or
      "category/1234".  */
-  if (! opened && ! get_pr (path, p, quiet))
-    return;
+  if ((query_format & FORMAT_FULL) && (! opened && ! (get_pr (path, p, quiet))))
+    return 0;
 
   if (is_daemon && !started)
     {
@@ -507,58 +515,148 @@ print_pr (path, p, opened)
       write_header (outfile, NUM_HEADER_ITEMS);
       fprintf (outfile, "\n");
       write_pr (outfile, NUM_PR_ITEMS);
+      return 1;
     }
-  else if (query_format & FORMAT_SQL)
+
+  if (!i)
+    return 0;
+  
+  if (query_format & FORMAT_SQL)
     {
+      char *t, *q, *tempstr;
+
+      tempstr = (char *) xmalloc (strlen (i->synopsis) + 1);
+      strcpy (tempstr, i->synopsis);
+      fprintf (outfile, "%-8.8s|%-16.16s|%-128.128s|%-3.3s|",
+	       i->number, i->category, disbar (tempstr), i->confidential);
+      xfree (tempstr);
+
       /* Trim `foo (Full Foo)' to just `foo'.  */
-      char *t, *q = (char *) strchr (field_value (RESPONSIBLE), ' ');
+      tempstr = (char *) xmalloc (strlen (i->responsible) + 1);
+      strcpy (tempstr, i->responsible);
+      q = (char *) strchr (tempstr, ' ');
       if (q != NULL)
 	*q = '\0';
 
-      fprintf (outfile, "%-8.8s|%-16.16s|%-128.128s|%-3.3s",
-	       field_value (NUMBER), field_value (CATEGORY),
-	       disbar(field_value (SYNOPSIS)),
-	       field_value (CONFIDENTIAL));
-      fprintf (outfile, "|%1.1d|%1.1d|%-16.16s|%1.1d|%1.1d|%-16.16s",
-	       sql_types (field_value (SEVERITY), Severity),
-	       sql_types (field_value (PRIORITY), Priority),
-	       field_value (RESPONSIBLE),
-	       sql_types (field_value (STATE), State),
-	       sql_types (field_value (CLASS), Class),
-	       field_value (SUBMITTER));
-      t = sql_time (field_value (ARRIVAL_DATE));
-      fprintf (outfile, "|%-18.18s|%-64.64s|%-64.64s|", t,
-	       disbar (field_value (ORIGINATOR)),
-	       field_value (RELEASE));
-      xfree (t);
-      if (strlen (field_value (LAST_MODIFIED)) == 0) {
-           fprintf (outfile, "%-18.18s|", "");
+      fprintf (outfile, "%1.1d|%1.1d|%-16.16s|%1.1d|%1.1d|%-16.16s|",
+	       sql_types (i->severity, Severity), sql_types (i->priority, Priority),
+	       tempstr,
+	       sql_types (i->state, State), sql_types (i->class, Class),
+	       i->submitter);
+      xfree (tempstr);
+      
+      if (strlen (i->arrival_date) == 0 || !strcmp (i->arrival_date, "0")) {
+        fprintf (outfile, "%-21.21s|", "");
       } else {
-           t = sql_time (field_value (LAST_MODIFIED));
-           fprintf (outfile, "%-18.18s|", t);
-           xfree (t);
+        t = sql_time_from_index (i->arrival_date);
+        fprintf (outfile, "%-21.21s|", t);
+        xfree (t);
       }
-      if (strlen (field_value (CLOSED_DATE)) == 0) {
-           fprintf (outfile, "%-18.18s|", "");
+      
+      fprintf (outfile, "%-64.64s|%-64.64s|", disbar (i->originator), i->release);
+      
+      if (strlen (i->last_modified) == 0 || !strcmp (i->last_modified, "0")) {
+        fprintf (outfile, "%-21.21s|", "");
       } else {
-           t = sql_time (field_value (CLOSED_DATE));
-           fprintf (outfile, "%-18.18s|", t);
-           xfree (t);
+        t = sql_time_from_index (i->last_modified);
+        fprintf (outfile, "%-21.21s|", t);
+        xfree (t);
       }
-    }
+      
+      if (strlen (i->closed_date) == 0 || !strcmp (i->closed_date, "0")) {
+        fprintf (outfile, "%-21.21s|", "");
+      } else {
+        t = sql_time_from_index (i->closed_date);
+        fprintf (outfile, "%-21.21s|", t);
+        xfree (t);
+      }
+#ifdef GNATS_RELEASE_BASED
+      fprintf (outfile, "%-64.64s|%-64.64s|", i->quarter, i->keywords);
+      if (strlen (i->date_required) == 0 || !strcmp (i->date_required, "0")) {
+        fprintf (outfile, "%-21.21s|", "");
+      } else {
+        t = sql_time_from_index (i->date_required);
+        fprintf (outfile, "%-21.21s|", t);
+        xfree (t);
+      }
+#endif
+    }  
+  else if (query_format & FORMAT_SQL2)
+    {
+      char *t, *q, *tempstr;
+
+      tempstr = (char *) xmalloc (strlen (i->synopsis) + 1);
+      strcpy (tempstr, i->synopsis);
+      fprintf (outfile, "%s|%s|%s|%s|",
+	       i->number, i->category, disbar (tempstr), i->confidential);
+      xfree (tempstr);
+
+      /* Trim `foo (Full Foo)' to just `foo'.  */
+      tempstr = (char *) xmalloc (strlen (i->responsible) + 1);
+      strcpy (tempstr, i->responsible);
+      q = (char *) strchr (tempstr, ' ');
+      if (q != NULL)
+	*q = '\0';
+
+      fprintf (outfile, "%d|%d|%s|%d|%d|%s|",
+	       sql_types (i->severity, Severity), sql_types (i->priority, Priority),
+	       tempstr,
+	       sql_types (i->state, State), sql_types (i->class, Class),
+	       i->submitter);
+      xfree (tempstr);
+      
+      if (strlen (i->arrival_date) == 0 || !strcmp (i->arrival_date, "0")) {
+        fprintf (outfile, "%s|", "");
+      } else {
+        t = sql_time_from_index (i->arrival_date);
+        fprintf (outfile, "%s|", t);
+        xfree (t);
+      }
+      
+      fprintf (outfile, "%s|%s|", disbar (i->originator), i->release);
+      
+      if (strlen (i->last_modified) == 0 || !strcmp (i->last_modified, "0")) {
+        fprintf (outfile, "%s|", "");
+      } else {
+        t = sql_time_from_index (i->last_modified);
+        fprintf (outfile, "%s|", t);
+        xfree (t);
+      }
+      
+      if (strlen (i->closed_date) == 0 || !strcmp (i->closed_date, "0")) {
+        fprintf (outfile, "%s|", "");
+      } else {
+        t = sql_time_from_index (i->closed_date);
+        fprintf (outfile, "%s|", t);
+        xfree (t);
+      }
+#ifdef GNATS_RELEASE_BASED
+      fprintf (outfile, "%s|%s|", i->quarter, i->keywords);
+      if (strlen (i->date_required) == 0 || !strcmp (i->date_required, "0")) {
+        fprintf (outfile, "%s|", "");
+      } else {
+        t = sql_time_from_index (i->date_required);
+        fprintf (outfile, "%s|", t);
+        xfree (t);
+      }
+#endif
+    }  
   else if (query_format & FORMAT_SUMM)
     {
-      char *s;
+      char *q, *tempstr;
+
+      /* Trim `foo (Full Foo)' to just `foo'.  */
+      tempstr = (char *) xmalloc (strlen (i->responsible) + 1);
+      strcpy (tempstr, i->responsible);
+      q = (char *) strchr (tempstr, ' ');
+      if (q != NULL)
+	*q = '\0';
 
-      s = (char *) strchr (pr[RESPONSIBLE].value, ' ');
-      if (s)
-	*s = '\0';
       fprintf (outfile,
 	       "%8s %-8.8s %-8.8s %-9.9s %-9.9s %-8.8s %-10.10s %s",
-	       pr[NUMBER].value, pr[RESPONSIBLE].value,
-	       pr[CATEGORY].value, pr[STATE].value, pr[SEVERITY].value,
-	       pr[PRIORITY].value, pr[SUBMITTER].value,
-	       pr[SYNOPSIS].value);
+	       i->number,   i->responsible, i->category, i->state, i->severity,
+	       i->priority, i->submitter,   i->synopsis);
+      xfree (tempstr);
     }
   else
     {
@@ -567,25 +665,26 @@ print_pr (path, p, opened)
       if (print_path)
 	fprintf (outfile, "%s:0:\n", path);
 
-      write_pr (outfile, NUMBER);
-      write_pr (outfile, CATEGORY);
-      write_pr (outfile, SYNOPSIS);
-      write_pr (outfile, CONFIDENTIAL);
-      write_pr (outfile, SEVERITY);
-      write_pr (outfile, PRIORITY);
-      write_pr (outfile, RESPONSIBLE);
-      write_pr (outfile, STATE);
-      write_pr (outfile, CLASS);
+      write_pr_from_index (outfile, NUMBER,        i->number);
+      write_pr_from_index (outfile, CATEGORY,      i->category);
+      write_pr_from_index (outfile, SYNOPSIS,      i->synopsis);
+      write_pr_from_index (outfile, CONFIDENTIAL,  i->confidential);
+      write_pr_from_index (outfile, SEVERITY,      i->severity);
+      write_pr_from_index (outfile, PRIORITY,      i->priority);
+      write_pr_from_index (outfile, RESPONSIBLE,   i->responsible);
+      write_pr_from_index (outfile, STATE,         i->state);
+      write_pr_from_index (outfile, CLASS,         i->class);
 #ifdef GNATS_RELEASE_BASED
-      write_pr (outfile, QUARTER);
-      write_pr (outfile, KEYWORDS);
-      write_pr (outfile, DATE_REQUIRED);
+      write_pr_from_index (outfile, QUARTER,       i->quarter);
+      write_pr_from_index (outfile, KEYWORDS,      i->keywords);
+      write_pr_from_index (outfile, DATE_REQUIRED, i->date_required);
 #endif
-      write_pr (outfile, SUBMITTER);
-      write_pr (outfile, ORIGINATOR);
-      write_pr (outfile, RELEASE);
-      write_pr (outfile, ARRIVAL_DATE);
-      write_pr (outfile, LAST_MODIFIED);
-      write_pr (outfile, CLOSED_DATE);
+      write_pr_from_index (outfile, SUBMITTER,     i->submitter);
+      write_pr_from_index (outfile, ARRIVAL_DATE,  i->arrival_date);
+      write_pr_from_index (outfile, CLOSED_DATE,   i->closed_date);
+      write_pr_from_index (outfile, LAST_MODIFIED, i->last_modified);
+      write_pr_from_index (outfile, ORIGINATOR,    i->originator);
+      write_pr_from_index (outfile, RELEASE,       i->release);
     }
+  return 1;
 }
